# Messwerte auslesen Sirius-Sensor Typ SEN55  Werner Becker 22-06-2024  Version 1.0 

import time
from machine import I2C, Pin

# Definiere die I2C-Adresse des SEN55-Sensors
SEN55_I2C_ADDRESS = 0x69  # I2C-Adresse

# Initialisiere I2C (I2C0 auf den Pins GP0 und GP1)
i2c = I2C(0, scl=Pin(13), sda=Pin(12), freq=400000)


devices = i2c.scan()

# Geräteadressen ausgeben


def sensirion_i2c_sleep(microseconds):
    time.sleep_us(microseconds)
    

if devices:
    print("I2C Geräte gefunden:", [hex(device) for device in devices])
else:
    print ("oh")    
    
    

def sensirion_i2c_read(bus, address, length):
    try:
        data = bus.readfrom(address, length)
        return list(data), 0
    except Exception as e:
        print(f"I2C read error: {e}")
        return [], -1

def sensirion_i2c_add_command_to_buffer(buffer, offset, command):
    """
    Fügt einen Befehl zum Buffer hinzu und gibt den neuen Offset zurück.
    """
    buffer[offset] = (command >> 8) & 0xFF  # Höheres Byte
    buffer[offset + 1] = command & 0xFF  # Niedrigeres Byte
    return offset + 2

def sensirion_i2c_write_data(bus, i2c_address, buffer, length):
    """
    Sendet die Daten im Buffer an die angegebene I2C-Adresse.
    """
    try:
        bus.writeto(i2c_address, bytes(buffer[:length]))
        return 0  # NO_ERROR
    except Exception as e:
        print(f"I2C write error: {e}")
        return -1  # Fehlercode

def sen5x_read_measured_values():
    NO_ERROR = 0
    DATA_LENGTH = 23  # Länge der zu lesenden Daten
    buffer = [0] * 2

    offset = sensirion_i2c_add_command_to_buffer(buffer, 0, 0x03C4)

    error = sensirion_i2c_write_data(i2c, SEN55_I2C_ADDRESS, buffer, offset)
    if error:
        return None, error

    sensirion_i2c_sleep(50000)

    data, error = sensirion_i2c_read(i2c, SEN55_I2C_ADDRESS, DATA_LENGTH)
    if error:
        return None, error

    mass_concentration_pm1p0 = (data[0] << 8) | data[1]
    mass_concentration_pm2p5 = (data[3] << 8) | data[4]
    mass_concentration_pm4p0 = (data[6] << 8) | data[7]
    mass_concentration_pm10p0 = (data[9] << 8) | data[10]
    ambient_humidity = (data[12] << 8) | data[13]
    ambient_temperature = (data[15] << 8) | data[16]
    voc_index = (data[18] << 8) | data[19]
    nox_index = (data[21] << 8) | data[22]

    measured_values = (
        mass_concentration_pm1p0, mass_concentration_pm2p5, 
        mass_concentration_pm4p0, mass_concentration_pm10p0, 
        ambient_humidity, ambient_temperature, 
        voc_index, nox_index
    )

    return measured_values, NO_ERROR

def sen5x_get_serial_number():
    NO_ERROR = 0
    DATA_LENGTH = 23  # Länge der zu lesenden Daten
    buffer = [0] * 2
    offset = sensirion_i2c_add_command_to_buffer(buffer, 0, 0xD033)
    error = sensirion_i2c_write_data(i2c, SEN55_I2C_ADDRESS, buffer, offset)
    if error:
        return None, error
    
    sensirion_i2c_sleep(50000)
    data, error = sensirion_i2c_read(i2c, SEN55_I2C_ADDRESS, DATA_LENGTH)
    if error:
        return None, error
    
    
    
    N1= str(chr(data[0])); N2= str(chr(data[1]))
    N3= str(chr(data[3])); N4= str(chr(data[4])) 
    N5= str(chr(data[6])); N6= str(chr(data[7]))
    N7= str(chr(data[9])); N8= str(chr(data[10]))
    N9= str(chr(data[12])); N10= str(chr(data[13]))
    N11= str(chr(data[15])); N12= str(chr(data[16])) 
    N13= str(chr(data[18])); N14= str(chr(data[19]))
    N15= str(chr(data[21])); N16= str(chr(data[22]))
    
    
    serial_number= (N1+N2+N3+N4+N5+N6+N7+N8+N9+N10+N11+N12+N13+N14+N15+N16)
    
    return serial_number, NO_ERROR



def sen5x_start_measurement():
    buffer = [0] * 2
    offset = sensirion_i2c_add_command_to_buffer(buffer, 0, 0x0021)

    error = sensirion_i2c_write_data(i2c, SEN55_I2C_ADDRESS, buffer, offset)
    return error

def sen5x_stop_measurement():
    buffer = [0] * 2
    offset = sensirion_i2c_add_command_to_buffer(buffer, 0, 0x0104)

    error = sensirion_i2c_write_data(i2c, SEN55_I2C_ADDRESS, buffer, offset)
    return error



# Starte die Messung
if __name__ == "__main__":
    sen5x_start_measurement()
    serial_number, error = sen5x_get_serial_number()
    print ("serial number: ", serial_number)
    print ()
    for _ in range(20):
        sensirion_i2c_sleep(1000000)
        measured_values, error = sen5x_read_measured_values()
        if error:
            print("Fehler beim Lesen der Messwerte")
            continue
        t = round((measured_values[5] / 200.0), 1)  # Temperatur auf eine Kommastelle gerundet
        f = round((measured_values[4] / 100.0), 1)  # Feuchte auf eine Kommastelle gerundet
        
        print("PM(1.0) =", str(measured_values[0]/10.0) + "μg/m³")
        print("PM(2.5) =", str(measured_values[1]/10.0) + "μg/m³")
        print("PM(4.0) =", str(measured_values[2]/10.0) + "μg/m³")
        print("PM(10) =", str(measured_values[3]/10.0) + "μg/m³")
        print("Feuchte =", str(f) + "%")
        print("Temperatur =", str(t) + "°C")
        print("VOC Index =", str(measured_values[6] / 10.0))
        print("NOX Index =", str(measured_values[7] / 10.0))
        print("---")
#sen5x_stop_measurement()
